@@ -21,6 +21,7 @@ from docutils import nodes
 from sphinx import addnodes
 from sphinx.builders.html import StandaloneHTMLBuilder
 
+
 _idpattern = re.compile(
     r'(?P<title>.+) (\((?P<id>[\w\.]+)( (?P<descr>\w+))?\))$')
 
@@ -126,9 +127,9 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
         for node in tocdoc.traverse(istoctree):
             sections.extend(self.write_toc(node))
 
-        if self.config.html_use_modindex:
-            item = section_template % {'title': _('Global Module Index'),
-                                       'ref': 'modindex.html'}
+        for indexname, indexcls, content, collapse in self.domain_indices:
+            item = section_template % {'title': indexcls.localname,
+                                       'ref': '%s.html' % indexname}
             sections.append(' '*4*4 + item)
         sections = '\n'.join(sections)
 
@@ -251,7 +252,7 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
                 shortname = shortname[:-2]
             id = '%s.%s' % (id, shortname)
         else:
-            id = descr = None
+            id = None
 
         if id:
             item = ' '*12 + '<keyword name="%s" id="%s" ref="%s"/>' % (
