@@ -41,7 +41,8 @@ class IfConfig(Directive):
         node.document = self.state.document
         node.line = self.lineno
         node['expr'] = self.arguments[0]
-        self.state.nested_parse(self.content, self.content_offset, node)
+        self.state.nested_parse(self.content, self.content_offset,
+                                node, match_titles=1)
         return [node]
 
 
